//************************************************************************
//************************************************************************
//* January 23, 2020 CONSTANT CURRENT LED DRIVER
//* M. Zinaddinov
//* This code senses current through LED and keeps it at an average value of
//* 350mA by modifying the duty cycle of a switching power MOSFET.
//* I intended another analog input for dimming potentiometer, however it has not
//* yet been implemented. I heavily relied on the material
//* provided by Prof. Mark M. Budnik in his "Microcontrollers and C Programming
//* Language (MSP430)" course.
//************************************************************************
//************************************************************************

#include <msp430.h>
#define ENABLE_PINS 0xFFFE
#define SMCLK 0x0200                               // Timer_A SMK source
#define UP 0x0010                                  // Timer_A UP mode

void ADC_SETUP(void);                              // Used to setup ADC12 peripheral

void Setup_GPIO_Pins(void) ;                       // Setup GPIO pins for SPI operation
void Setup_Clocks(void) ;                          // Setup clocks for synch operation
void Setup_SPI_B0(void) ;                          // Setup UCSI type B, number 0 for SPI
void Send_SPI_B0_16(unsigned int DataToSend);      // Sends 16-bit data on SPI

main()
{
    WDTCTL  = WDTPW | WDTHOLD; // Stop WDT
    PM5CTL0 = ENABLE_PINS;     // Enable inputs and outputs
    P1DIR   = BIT0;            // Set P1.0 to output (gate)
    P1SEL0  = BIT0;            // Gives Timer0 control over P1.0

    TA0CCTL1 = OUTMOD_3;       // Use Timer mode 3
                               // Timer starts at 0, P1.0 starts LO
                               // When timer reaches TA0CCR1, P1.0 goes HI
                               // When timer reaches TA0CCR0, P1.0 goes LO
                               // Count starts over at 0 with P1.0 LO
    TA0CCR1 = 10; // P1.0 LO from 0 - 90
    TA0CCR0 = 100; // P1.0 HI to 100

    //SETUP SPI
    Setup_GPIO_Pins(); // Setup GPIO pins for SPI communication
    Setup_Clocks();    // Setup clocks for synch communication
    Setup_SPI_B0();    // Setup UCSI port type B, number 0 as SPI

    //SETUP ADC+Interrupt
    ADC_SETUP(); // Sets up ADC peripheral
    ADC12IER0 = ADC12IE0; // Enable ADC interrupt
    _BIS_SR(GIE); // Activate interrupts

    TA0CTL = SMCLK | UP; // Count up to TA0CCR0 at 1us/count
    ADC12CTL0 = ADC12CTL0 | ADC12ENC; // Enable conversion
    ADC12CTL0 = ADC12CTL0 | ADC12SC; // Start conversion

    while(1);
}

//************************************************************************
//* ADC12 Interrupt Service Routine***************************************
//************************************************************************

#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
    #define ADC12_P84 0x0007 // Use input P8.4 for analog input (Curr_sense)
    #define ADC12_P92 0x000A // Use input P9.2 for analog input (Dim_sense)

    static unsigned char input = 84; // input = 84 if P8.4 sampled
                                     //       = 92 if P9.2 sampled

    if(input == 84) // If sample was from P8.4
    {
        Send_SPI_B0_16(ADC12MEM0);

        if (ADC12MEM0 > 0x1C0) // If input > 0.35V (350mA through 1 ohm)
        {
            if (TA0CCR1 > 10) TA0CCR1 -= 2;

        }
        else
        {
            if (TA0CCR1 < 95) TA0CCR1 += 1;

        }

        input = 92; // Next sample from P9.2

        ADC12CTL0 = ADC12CTL0 & (~ADC12ENC); // Need to disable peripheral
        ADC12MCTL0 = ADC12_P92; // to change to input P9.2
    }

    else // Else, sample was from P9.2 (I want to use this to sense potentiometer position for dimming)
    {

        //DIM_POT goes here

        input = 84; // Next sample from P8.4

        ADC12CTL0 = ADC12CTL0 & (~ADC12ENC); // Need to disable peripheral
        ADC12MCTL0 = ADC12_P84; // to change to input P8.4
    }

    ADC12CTL0 = ADC12CTL0 | ADC12ENC; // Re-enable conversion
    ADC12CTL0 = ADC12CTL0 | ADC12SC; // Start next conversion
}

//************************************************************************
//* Configure Analog-to-Digital Converter peripheral**********************
//************************************************************************
void ADC_SETUP(void)
{
    #define ADC12_SHT_16 0x0200 // 16 clock cycles for sample and hold
    #define ADC12_ON 0x0010 // Used to turn ADC12 peripheral on
    #define ADC12_SHT_SRC_SEL 0x0204 // Selects source for sample & hold (ACLK)
    #define ADC12_12BIT 0x0020 // Selects 12-bits of resolution

    #define ADC12_P84 0x0007 // Use input P8.4 for analog input
    #define ADC12_P92 0x000A // Use input P9.2 for analog input

    ADC12CTL0 = ADC12_SHT_16 | ADC12_ON ; // Turn on, set sample & hold time
    ADC12CTL1 = ADC12_SHT_SRC_SEL; // Specify sample & hold clock source
    ADC12CTL2 = ADC12_12BIT; // 12-bit conversion results
    ADC12MCTL0 = ADC12_P84; // P8.4 is analog input
}

//////////////////////////////////////////////////////////////////////////// SPI
////////////////////////////////////////////////////////////////////////////

//******************************************************************************************
//* Setup general purpose input and output pins for SPI operation
//* We will not use the SOMI pin in this application, but we wanted to show you how to
//* initialize it.
//******************************************************************************************

void Setup_GPIO_Pins(void)
{
     PM5CTL0 = ENABLE_PINS;  // Enables use of inputs and outputs
     P1SEL0 = P1SEL0 | BIT4; // P1.4 ==> SPI clock signal
     P1SEL0 = P1SEL0 | BIT6; // P1.6 ==> SPI Slave In / Master Out (SIMO)
     P1SEL0 = P1SEL0 | BIT7; // P1.7 ==> SPI Slave Out / Master In (SOMI)
     P2OUT = BIT0;           // Want CS to start HI to avoid possible glitches
     P2DIR = BIT0;           // Make CS pin an output
     PJSEL0 = BIT4 | BIT5;   // Enable some clock pins for SPI's synch operation
}
//********************************************************************************************
//* Will enable us to use a new clock signal for our SPI
//********************************************************************************************
void Setup_Clocks(void)
{
     CSCTL0_H = CSKEY >> 8; // Unlock CS registers
     CSCTL1 = DCOFSEL_0;    // Set DCO to 1MHz
     CSCTL2 = SELA__LFXTCLK | SELS__DCOCLK | SELM__DCOCLK; // ACLK: 32.768kHz
                                                           // SMCLK will use DCO (1MHz)
                                                           // MCLK will use DCO (1MHz)
     CSCTL3 = DIVA__1 | DIVS__1 | DIVM__1; // Do change any frequencies
     CSCTL4 = CSCTL4 & (~LFXTOFF);         // Enable 32.768kHz crystal
     do                                    // Wait here until 32.768kHz
     {                                     // clock is ready
         CSCTL5 = CSCTL5 & (~LFXTOFFG);    // Clear 32.768kHz fault flags
         SFRIFG1 = SFRIFG1 & (~OFIFG);     // Clear 32.768kHz fault flags
     }while (SFRIFG1&OFIFG);               // Test 32.768kHz oscillator
                                           // fault flag
     CSCTL0_H = 0;                         // Lock CS registers
}
//*********************************************************************************
// Configure USCI_B0 for SPI operation
//*********************************************************************************
void Setup_SPI_B0(void)
{
     UCB0CTLW0 = UCSWRST;                  // Puts Universal Communication (UC) peripheral
                                           // into SoftWare (SW) ReSeT (RST) -- UCSWRST
                                           // to Disables USCI so it can be setup. Most of
                                           // these modifications can only be made when the
                                           // USCI is disabled
     UCB0CTLW0 = UCB0CTLW0 | UCMST;        // Microcontroller will be the master, so
                                           // Master (M) mode SelecTed (ST) -- UCMST
     UCB0CTLW0 = UCB0CTLW0 | UCSYNC;       // SPI needs clock so SYNChronous mode selected
     UCB0CTLW0 = UCB0CTLW0 | UCCKPH;       // Specifies ClocK (CK) PHase (PH). UCCKPH: data
                                           // captured on first CLK edge and changed on next
     UCB0CTLW0 = UCB0CTLW0 | UCMSB;        // Specifies the Most Significant Bit of data will
                                           // be transimitted first -- UCMSB
     UCB0CTLW0 = UCB0CTLW0 | UCSSEL__ACLK; // clock Source (S) is SELected (SEL) to be ACLK
     UCB0BR1 = 0x00;                       // Sets up clock divider to slow data transmission
     UCB0BR0 = 0x02;                       // Div by 2 : CLK is 16.4kHz, data bit 61us wide
     UCB0CTLW0 = UCB0CTLW0 & (~UCSWRST);   // Takes Universal Communication (UC) peripheral
                                           // out of SoftWare (SW) ReSeT (RST) -- UCSWRST
                                           // since we are now done setting up the peripheral
}
//*********************************************************************************
//* Send 16-bit DataToSend on SPI port in two, 8-bit pieces
//*********************************************************************************
void Send_SPI_B0_16(unsigned int DataToSend)
{
    while ( (UCB0STATW & UCBUSY) != 0); // This checks the UCBUSY bit in the UCB0STATW
                                        // (UCB0 STATus Word) register. The bit will
                                        // be HI if the peripheral is sending or
                                        // receiving. Therefore, wait here until the
                                        // UCBUSY bit goes LO.
    P2OUT = 0x00;                       // Pull the Chip Select line LO to tell the DAC
                                        // to pay attention
    UCB0TXBUF = (DataToSend>>8) ;       // Shift 8 most significant bits over to lower 8
                                        // slots and then load into the transmit (TX)
                                        // BUFfer to send
    while ( (UCB0IFG & UCTXIFG) == 0 ); // Test the TX IFG flag to see when the 8 most
                                        // significant bits have been sent
    UCB0TXBUF = (DataToSend & 0xFF) ;   // Clear out 8 most significant bits so we only
                                        // send the 8 least signifcant bits
    while ( (UCB0STATW & UCBUSY) != 0 ); // Wait until signal that everything is complete
    P2OUT = BIT0;                       // Raise chip select line to HI to tell the DAC
                                        // to stop listening
}
